@@ -4,6 +4,8 @@
 #define MIN_CAM_DISTANCE      5.0f  
 #define MAX_CAM_DISTANCE     10.0f  // Was 15
 
+int player=0;
+
 int finishing_position = -1 ;
 
 static ulClock ck2 ;
@@ -29,13 +31,7 @@ HerringInstance herring [ MAX_HERRING ] ;
 
 sgCoord steady_cam ;
  
-char *player_files [] =
-{
-  "tuxkart.ac" , "geekokart.ac",
-  "bsodkart.ac", "gownkart.ac",
-  NULL
-} ;
-
+char player_files [ NUM_KARTS ][ 256 ] ;
 
 char *traffic_files [] =
 {
@@ -98,9 +94,9 @@ sgCoord fixedpos [ MAX_FIXED_CAMERA ] =
 Level level ;
 
 
-void load_players ()
+void load_players ( char *fname )
 {
-  ssgEntity *obj ;
+  ssgEntity *obj;
 
   ssgEntity *pobj1 = ssgLoad ( parachute_file, loader_opts ) ;
   ssgEntity *pobj2 = ssgLoad ( magnet_file   , loader_opts ) ;
@@ -114,24 +110,70 @@ void load_players ()
 
   ssgEntity *pobj5 = ttt ;
   int i ;
+ 
+  FILE *fd = fopen ( fname, "r" ) ;
 
-  for ( i = 0 ; player_files [ i ] != NULL && i < NUM_KARTS ; i++ )
+  if ( fd == NULL )
+  {
+    fprintf ( stderr, "tuxkart: Can't open '%s':\n", fname ) ;
+    exit ( 1 ) ;
+  }
+
+  num_karts = 0 ;
+
+  while ( num_karts < NUM_KARTS )
+  {
+    if ( fgets ( player_files [ num_karts ], 256, fd ) != NULL )
+    {
+      if ( player_files [ num_karts ][ 0 ] <= ' ' ||
+           player_files [ num_karts ][ 0 ] == '#' )
+        continue ;
+
+      /* Trim off the '\n' */
+
+      int len = strlen ( player_files [ num_karts ] ) - 1 ;
+
+      if ( player_files [ num_karts ][ len ] <= ' ' )
+        player_files [ num_karts ][ len ] = '\0' ;
+
+fprintf(stderr,"Kart %d == '%s'\n", num_karts, player_files [ num_karts ] ) ;
+
+      num_karts++ ;
+    }
+    else
+      break ;
+  }
+ 
+  fclose ( fd ) ;
+
+  if ( player >= num_karts )
+    player = 0 ;
+
+  for ( i = 0 ; i < num_karts ; i++ )
   {
     ssgRangeSelector *lod = new ssgRangeSelector ;
     float r [ 2 ] = { -10.0f, 100.0f } ;
+    int kart_id ;
 
-    obj = ssgLoad ( player_files [ i ], loader_opts ) ;
-
-    lod -> addKid ( obj   ) ;
-    lod -> setRanges ( r, 2  ) ;
+    if ( i == 0 )
+      kart_id = player ;
+    else
+    if ( i == player )
+      kart_id = 0 ;
+    else
+      kart_id = i ;
  
+    obj = ssgLoad ( player_files [ kart_id ], loader_opts ) ;
+
+    lod -> addKid ( obj ) ;
+    lod -> setRanges ( r, 2 ) ;
+    
     kart[i]-> getModel() -> addKid ( lod ) ;
     kart[i]-> addAttachment ( pobj1 ) ;
     kart[i]-> addAttachment ( pobj2 ) ;
     kart[i]-> addAttachment ( pobj3 ) ;
     kart[i]-> addAttachment ( pobj4 ) ;
     kart[i]-> addAttachment ( pobj5 ) ;
-    num_karts = i + 1 ;
   }
 
   for ( i = 0 ; i < NUM_PROJECTILES ; i++ )
@@ -195,6 +237,9 @@ static void herring_command ( char *s, char *str )
 void load_track ( ssgBranch *trackb, char *fname )
 {
   FILE *fd = fopen ( fname, "r" ) ;
+  char playersfname [ 256 ] ;
+
+  strcpy ( playersfname, "data/players.dat" ) ;
 
   if ( fd == NULL )
   {
@@ -220,6 +265,11 @@ void load_track ( ssgBranch *trackb, char *fname )
 
     char htype = '\0' ;
 
+    if ( sscanf ( s, "PLAYERS \"%[^\"]\"", fname ) == 1 )
+    {
+      strcpy ( playersfname, fname ) ;
+    }
+    else
     if ( sscanf ( s, "MUSIC \"%[^\"]\"", fname ) == 1 )
     {
       sound -> change_track ( fname ) ;
@@ -342,12 +392,14 @@ void load_track ( ssgBranch *trackb, char *fname )
       fprintf ( stderr, "  %s\n", s ) ;
       exit ( 1 ) ;
     }
-
   }
+
   fclose ( fd ) ;
 
   sgSetVec3  ( steady_cam.xyz, 0.0f, 0.0f, 0.0f ) ;
   sgSetVec3  ( steady_cam.hpr, 0.0f, 0.0f, 0.0f ) ;
+
+  load_players ( playersfname ) ;
 }
 
 
@@ -474,11 +526,11 @@ int tuxkart_main ( int num_laps, char *level_name )
   gfx   = new GFX ;
 
   sound = new SoundSystem ;
-/*
-  sound -> change_track ( "mods/mfarmer.mod" ) ;
-*/
+  // sound -> change_track ( "mods/Boom_boom_boom.mod" ) ;
   gui   = new GUI ;
 
+  pwSetCallbacks ( keystroke, mousefn, motionfn, reshape, NULL ) ;
+
   ssgModelPath   ( "models" ) ;
   ssgTexturePath ( "images" ) ;
 
@@ -534,8 +586,10 @@ int tuxkart_main ( int num_laps, char *level_name )
   sprintf ( fname, "data/%s.loc", trackname ) ;
 
   load_track ( trackb, fname ) ;
-  load_players () ;
 
+fprintf ( stderr, "READY TO RACE!!\n" ) ;
+
+  tuxKartMainLoop () ;
   return TRUE ;
 }
 
@@ -676,40 +730,42 @@ void updateWorld ()
 
 void tuxKartMainLoop ()
 {
-
-  if ( ! gui -> isPaused () )
+  while ( 1 )
   {
-    fclock->update () ;
-  ck2.update() ; tt[0] = ck2.getDeltaTime()*1000.0f ;
-    updateWorld () ;
+    if ( ! gui -> isPaused () )
+    {
+      fclock->update () ;
+      ck2.update() ; tt[0] = ck2.getDeltaTime()*1000.0f ;
+      updateWorld () ;
 
-    for ( int i = 0 ; i < MAX_HERRING ; i++ )
-      if ( herring [ i ] . her != NULL )
-        herring [ i ] . update () ;
+      for ( int i = 0 ; i < MAX_HERRING ; i++ )
+        if ( herring [ i ] . her != NULL )
+          herring [ i ] . update () ;
 
-    silver_h -> update () ;
-    gold_h   -> update () ;
-    red_h    -> update () ;
-    green_h  -> update () ;
+      silver_h -> update () ;
+      gold_h   -> update () ;
+      red_h    -> update () ;
+      green_h  -> update () ;
 
-    update_hooks () ;
-  }
-  else
-  {
-  ck2.update() ; tt[0] = ck2.getDeltaTime()*1000.0f ;
-  ck2.update() ; tt[1] = ck2.getDeltaTime()*1000.0f ;
-  ck2.update() ; tt[2] = ck2.getDeltaTime()*1000.0f ;
-  ck2.update() ; tt[3] = ck2.getDeltaTime()*1000.0f ;
-  }
+      update_hooks () ;
+    }
+    else
+    {
+      ck2.update() ; tt[0] = ck2.getDeltaTime()*1000.0f ;
+      ck2.update() ; tt[1] = ck2.getDeltaTime()*1000.0f ;
+      ck2.update() ; tt[2] = ck2.getDeltaTime()*1000.0f ;
+      ck2.update() ; tt[3] = ck2.getDeltaTime()*1000.0f ;
+    }
 
 /*track  -> update () ; */
 
-  ck2.update() ; tt[4] = ck2.getDeltaTime()*1000.0f ;
-  gfx    -> update () ;
-  gui    -> update () ;
-  sound  -> update () ;
-  gfx    -> done   () ;  /* Swap buffers! */
-  ck2.update() ; tt[5] = ck2.getDeltaTime()*1000.0f ;
+    ck2.update() ; tt[4] = ck2.getDeltaTime()*1000.0f ;
+    gfx    -> update () ;
+    gui    -> update () ;
+    sound  -> update () ;
+    gfx    -> done   () ;  /* Swap buffers! */
+    ck2.update() ; tt[5] = ck2.getDeltaTime()*1000.0f ;
+  }
 }
 
 
